==> v1.6.x.zip: opentelemetry-java-instrumentation/instrumentation/servlet/servlet-common/javaagent/src/main/java/io/opentelemetry/javaagent/instrumentation/servlet/AsyncRequestCompletionListener.java <==
/*
 * Copyright The OpenTelemetry Authors
 * SPDX-License-Identifier: Apache-2.0
 */

package io.opentelemetry.javaagent.instrumentation.servlet;

import io.opentelemetry.context.Context;
import io.opentelemetry.instrumentation.api.instrumenter.Instrumenter;
import io.opentelemetry.instrumentation.servlet.ServletAsyncListener;
import java.util.concurrent.atomic.AtomicBoolean;

public class AsyncRequestCompletionListener<REQUEST, RESPONSE>
    implements ServletAsyncListener<RESPONSE> {
  private final ServletHelper<REQUEST, RESPONSE> servletHelper;
  private final Instrumenter<ServletRequestContext<REQUEST>, ServletResponseContext<RESPONSE>>
      instrumenter;
  private final ServletRequestContext<REQUEST> requestContext;
  private final Context context;
  private final AtomicBoolean responseHandled = new AtomicBoolean();

  public AsyncRequestCompletionListener(
      ServletHelper<REQUEST, RESPONSE> servletHelper,
      Instrumenter<ServletRequestContext<REQUEST>, ServletResponseContext<RESPONSE>> instrumenter,
      ServletRequestContext<REQUEST> requestContext,
      Context context) {
    this.servletHelper = servletHelper;
    this.instrumenter = instrumenter;
    this.requestContext = requestContext;
    this.context = context;
  }

  @Override
  public void onComplete(RESPONSE response) {
    if (responseHandled.compareAndSet(false, true)) {
      ServletResponseContext<RESPONSE> responseContext =
          new ServletResponseContext<>(response, null);
      instrumenter.end(context, requestContext, responseContext, null);
    }
  }

  @Override
  public void onTimeout(long timeout) {
    if (responseHandled.compareAndSet(false, true)) {
      RESPONSE response = servletHelper.getAsyncListenerResponse(requestContext.request());
      ServletResponseContext<RESPONSE> responseContext =
          new ServletResponseContext<>(response, null);
      responseContext.setTimeout(timeout);
      instrumenter.end(context, requestContext, responseContext, null);
    }
  }

  @Override
  public void onError(Throwable throwable, RESPONSE response) {
    if (responseHandled.compareAndSet(false, true)) {
      ServletResponseContext<RESPONSE> responseContext =
          new ServletResponseContext<>(response, throwable);
      instrumenter.end(context, requestContext, responseContext, throwable);
    }
  }
}

==> v2.15.x.zip: opentelemetry-java-instrumentation/instrumentation/servlet/servlet-common/javaagent/src/main/java/io/opentelemetry/javaagent/instrumentation/servlet/AsyncRequestCompletionListener.java <==
/*
 * Copyright The OpenTelemetry Authors
 * SPDX-License-Identifier: Apache-2.0
 */

package io.opentelemetry.javaagent.instrumentation.servlet;

import io.opentelemetry.context.Context;
import io.opentelemetry.instrumentation.api.instrumenter.Instrumenter;
import java.util.concurrent.atomic.AtomicBoolean;

public class AsyncRequestCompletionListener<REQUEST, RESPONSE>
    implements ServletAsyncListener<RESPONSE> {
  private final ServletHelper<REQUEST, RESPONSE> servletHelper;
  private final Instrumenter<ServletRequestContext<REQUEST>, ServletResponseContext<RESPONSE>>
      instrumenter;
  private final ServletRequestContext<REQUEST> requestContext;
  private final Context context;
  private final AtomicBoolean responseHandled = new AtomicBoolean();

  public AsyncRequestCompletionListener(
      ServletHelper<REQUEST, RESPONSE> servletHelper,
      Instrumenter<ServletRequestContext<REQUEST>, ServletResponseContext<RESPONSE>> instrumenter,
      ServletRequestContext<REQUEST> requestContext,
      Context context) {
    this.servletHelper = servletHelper;
    this.instrumenter = instrumenter;
    this.requestContext = requestContext;
    this.context = context;
  }

  @Override
  public void onComplete(RESPONSE response) {
    if (responseHandled.compareAndSet(false, true)) {
      ServletResponseContext<RESPONSE> responseContext = new ServletResponseContext<>(response);
      Throwable throwable = servletHelper.getAsyncException(context);
      instrumenter.end(context, requestContext, responseContext, throwable);
    }
  }

  @Override
  public void onTimeout(long timeout) {
    if (responseHandled.compareAndSet(false, true)) {
      RESPONSE response = servletHelper.getAsyncListenerResponse(context);
      ServletResponseContext<RESPONSE> responseContext = new ServletResponseContext<>(response);
      responseContext.setTimeout(timeout);
      Throwable throwable = servletHelper.getAsyncException(context);
      instrumenter.end(context, requestContext, responseContext, throwable);
    }
  }

  @Override
  public void onError(Throwable throwable, RESPONSE response) {
    if (responseHandled.compareAndSet(false, true)) {
      ServletResponseContext<RESPONSE> responseContext = new ServletResponseContext<>(response);
      instrumenter.end(context, requestContext, responseContext, throwable);
    }
  }
}
